package plaver.com.sun.dtv.ui.event.UserInputEventManager;

import javax.microedition.xlet.XletStateChangeException;

import plaver.dtvtest.DTVTest;
import plaver.dtvtest.DTVTestLog;


import com.sun.dtv.lwuit.Form;
import com.sun.dtv.resources.ScarceResource;
import com.sun.dtv.resources.ScarceResourceListener;
import com.sun.dtv.resources.TimeoutException;
import com.sun.dtv.ui.Device;
import com.sun.dtv.ui.Screen;
import com.sun.dtv.ui.event.RemoteControlEvent;
import com.sun.dtv.ui.event.UserInputEvent;
import com.sun.dtv.ui.event.UserInputEventListener;
import com.sun.dtv.ui.event.UserInputEventManager;

/**
 * Testa mtodos que precisam da interao do usurio. A partir desta classe 
 * sero avaliados os seguintes mtodos da classe UserInputEventManager:
 *
 *<ol>
 *<li>{@link UserInputEventManager#addUserInputEventListener(com.sun.dtv.ui.event.UserInputEventListener, UserInputEvent)}</li>
 *<li>{@link UserInputEventManager#removeUserInputEventListener(com.sun.dtv.ui.event.UserInputEventListener)}</li>
 *</ol>
 *
 *<p>Tambm ser verificado todas as teclas de interatividade de um controle remoto de TV Digital</p>
 *
 * @author Ezequiel da Silva
 * @author Leandro Henrique
 * @author Wennio Oliveira
 *
 * @see UserInputEventManager
 */
public class RemoveUserInputEventManagerTest implements DTVTest, UserInputEventListener, ScarceResourceListener, Runnable {

	private DTVTestLog dtvTestLog;
	
	private UserInputEventManager eventManager;
	private RemoteControlEvent eventoRCE;
	private Device device;
	private Screen currentScreen;
	

	public void setLog(DTVTestLog log) {
		this.dtvTestLog = log;
	}
	
	public void init() {
		device = Device.getInstance();
		currentScreen = device.getDefaultScreen();
		eventManager = UserInputEventManager.getUserInputEventManager(currentScreen);
	}

	public void start() {
		/* Determina evento da classe RemoteControlEvent */
		try {
			determinaEventoRCE();
			testaInteracao();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		//Aqui comea os testes
	}	
	
	/**
	 * Testa todos os mtodos da classe {@link UserInputEventManager} atravs
	 * da interao do usurio com a aplicao.
	 */
	private void testaInteracao() {
		
		/* Verifica se a aplicao est respondendo a interao do usurio */
		eventManager.addUserInputEventListener(this, eventoRCE);
		eventManager.removeUserInputEventListener(this);
		
		try {
			eventoRCE.reserve(true, -1, this);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (TimeoutException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * Instancia um evento de usurio atravs da classe {@link RemoteControlEvent RemoteControlEvent} e faz
	 * a sua reserva, definindo o tempo em que o evento poder ser tratado e sua respectiva classe tratadora.
	 */
	private void determinaEventoRCE() {
		eventoRCE = new RemoteControlEvent(null, 		  //Componente que originou o evento
        		RemoteControlEvent.KEY_RELEASED,          //Tipo de evento
        		0,										  //Tempo que o evento ocorreu
        		0,										  //Modificadores
        		RemoteControlEvent.VK_COLORED_KEY_0,	  //Recurso utilizado pelo evento (VK_UNDEFINED no est implementado)
        		RemoteControlEvent.CHAR_UNDEFINED		  //Caracter unicode que gerou o evento
        );
	}

	public synchronized void userInputEventReceived(UserInputEvent inputEvent) {
		dtvTestLog.success();
		
		com.sun.dtv.ui.event.KeyEvent e = (com.sun.dtv.ui.event.KeyEvent) inputEvent;

		/* Os eventos de release (soltar o boto) no sero tratados */
//		if(e.getID() == KeyEvent.KEY_RELEASED)
			
	}


	public void pause() {
		// TODO Auto-generated method stub
	}

	public void destroy() {

	}

	public void pauseXlet() {
		// TODO Auto-generated method stub
		
	}

	public void destroyXlet(boolean unconditional)
			throws XletStateChangeException {
		// TODO Auto-generated method stub
	}

	public boolean releaseRequested(ScarceResource resource) {
		// TODO Auto-generated method stub
		return false;
	}

	public void releaseForced(ScarceResource resource) {
		// TODO Auto-generated method stub
	}

	public void released(ScarceResource resource) {
		// TODO Auto-generated method stub
	}

	public void run() {
		// TODO Auto-generated method stub
	}

	public void setForm(Form form) {
		// TODO Auto-generated method stub
		
	}

}
