package plaver.dtvtest;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import javax.microedition.xlet.XletContext;

import com.sun.dtv.lwuit.Form;

/**
 * Classe responsvel por controlar a execuo dos testes. Tem como objetivo
 * reduzir o acoplamento entre os testes e o Xlet no qual eles estao anexados.
 *
 * @author GILBERTO FARIAS
 * @author Bruno Pessoa
 * 
 */
public class DTVTestController implements DTVTestListener {
	
	private Thread ctrlThread;    
    private DTVTestLog dtvTestLog;
    private DTVTest test;  
    private List listeners;
    
    private int appID;
    private int executionModeID;
    private long timeOut;
    
    private static Form 	   form;
    private static List 	   controllerList;
    public  static XletContext contextXlet;
    
	private DTVTestController(DTVTest app, int appID, int executionModeID, long timeOut) {
        this.test = app;
        this.appID = appID;
        this.executionModeID = executionModeID; 
        this.dtvTestLog = new DTVTestLog(timeOut, executionModeID);   

        listeners = new ArrayList();
    }  
    
	/**
	 * 
	 * Cria uma lista de aplicaes a serem testadas a patir de um arquivo de texto
	 * 
	 * @param filePath - Diretrio com a lista de aplicaes
	 * @return lista de aplicao que implementam {@link DTVTest} a serem testadas
	 */
    public static List createTestSuit(String filePath, Form form, XletContext contextXlet) {    
    	
    	DTVTestController.contextXlet = contextXlet;
    	DTVTestController.form = form;
    	
    	DTVTestController controller = null;    	
    	BufferedReader reader = null;
    	
    	int appID = 0;
    	long timeOut = 0;
    	
    	String executionMode;
    	int    executionModeID = 0;
    	DTVTest app = null;
    	
		try {
			reader = new BufferedReader(new FileReader(filePath));	
			controllerList = new ArrayList();
			
			StringTokenizer st;
			String timeoutDefault;
	        String line;	        
	        
	        reader.readLine(); //Read [timeout_default]
	        reader.readLine(); //Read [classname] [id] [auto/interactive] [timeout]
	        
	        timeoutDefault = reader.readLine();
	        st = new StringTokenizer(timeoutDefault);
	        
	        long defaultTimeOut = Long.parseLong(st.nextToken());
	        
	        while ((line = reader.readLine()) != null ) {
	        	st = new StringTokenizer(line);
	        	
	        	while (st.hasMoreTokens()) {
	        		
		        	Class dtClass = Class.forName(st.nextToken());
		        	       
		        	app = (DTVTest) dtClass.newInstance();
		        	
 				   	// Instancia o controller e o insere na lista
 				   	appID = Integer.parseInt(st.nextToken());
 				   
 				   	executionMode = st.nextToken();
 				   
 				   	if(executionMode.equals("auto")) {
 				   		executionModeID = DTVTest.AUTO;
 				   	} else if(executionMode.equals("interactive")) {
 				   		executionModeID = DTVTest.INTERACTIVE;
 				   	} else {
 				   		throw new Exception("Erro: Enter auto or interactive execution mode");
 				   	}
 				  
 				   	if(st.hasMoreElements())
 				   		timeOut = Long.parseLong(st.nextToken());
 				   	else
 				   		timeOut = defaultTimeOut; 
 				   
 				   	controller = new DTVTestController(app, appID, executionModeID, timeOut);				   
 				   	controllerList.add(controller);	 				        		
 				   
	        	}
	        	
	        } 	
	        
		} catch (Exception e) {
			e.printStackTrace();
		} finally {

			try {
				reader.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			
		}
		
        return controllerList;
    }   
    
    public void addDTVTestEventListener(DTVTestListener listener) {
        listeners.add(listener);
    }

    public void removeDTVTestEventListener(DTVTestListener listener) {
        listeners.remove(listener);
    }

    
    private void notifyEventReceived(final DTVTestEvent e) {
    	
    	Iterator it = listeners.iterator();
    	
        while (it.hasNext()) {
        	final DTVTestListener l = (DTVTestListener) it.next();        	
        	
        	new Thread() {
                public void run() {
                    l.testLogEventReceived(e);
                }
            }.start();
            
        }
    }    
        
    /**
     * Inicia a execuo do teste em uma diferente linha de execuo
     */
    public void start() {
    	
        dtvTestLog.addDTVTestEventListener(this);
                
        test.setForm(form);
        test.setLog(dtvTestLog);
                
        ctrlThread = new Thread() {            
            public void run() {  
            	
            	/* Visual application must not have timeout */
            	if(executionModeID == DTVTest.AUTO)
            		dtvTestLog.start();
            	
            	try {
            		
            		test.init();
            		test.start();
            		
            	} catch(Exception e) {
            		
            		if(dtvTestLog.getStatus() != DTVTestEvent.TEST_VISUAL_APP)
            			dtvTestLog.failure();
            	}

            }
        };        
        
        ctrlThread.start(); 

    }

    public void testLogEventReceived(DTVTestEvent e) {    	
    	
    	//<await !ctrlThread.isAlive()>
        while (ctrlThread.isAlive()) {
        	try {        		
        		
        		/* Se algum evento for gerado, tentar interromper threads em espera */
        		if(dtvTestLog.getStatus() != 0) {
        			ctrlThread.interrupt();
        		}
        		
				Thread.sleep(300);
				
			} catch (InterruptedException e1) {
				System.exit(1);
			}        	
        }         	
        
        System.out.print(test.getClass().getName());
        
        if(dtvTestLog.getStatus() == DTVTestEvent.TEST_SUCCESS)
        	System.out.println(": SUCESSO");
        else if(dtvTestLog.getStatus() == DTVTestEvent.TEST_FAILURE)
        	System.out.println(": FALHA");
        else if(dtvTestLog.getStatus() == DTVTestEvent.TEST_TIMEOUT)
        	System.out.println(": TIMEOUT");
        else
        	System.out.println(": APP_VISUAL");
        	
    	test.destroy();    	
    	dtvTestLog.destroy();   	
    	
    	try {
			Thread.sleep(500);
		} catch (InterruptedException a) {
			a.printStackTrace();
		}
    	
    	notifyEventReceived(e);        
    }		

	public int getAppID() {
		return appID;
	}

	public int getExecutionModeID() {
		return executionModeID;
	}

	public long getTimeOut() {
		return timeOut;
	}
	
	public static int getQtdAutoApps() {
		List controllerList = DTVTestController.controllerList;
		int cont = 0;
		
		if(DTVTestController.controllerList == null)
			return 0;
		for(int i = 0; i < controllerList.size(); i++) {
			if( ((DTVTestController) controllerList.get(i)).getExecutionModeID() == DTVTest.AUTO)
				cont++;
		}
		
		return cont;
		
	}
	
	public static int getQtdInteractiveApps() {
		List controllerList = DTVTestController.controllerList;
		int cont = 0;
		
		if(DTVTestController.controllerList == null)
			return 0;
		
		for(int i = 0; i < controllerList.size(); i++) {
			if( ((DTVTestController) controllerList.get(i)).getExecutionModeID() == DTVTest.INTERACTIVE)
				cont++;
		}
		
		return cont;
	}

}
