package plaver.dtvtest;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * Classe auxiliar para o desenvolvimento de aplicaes de teste para avaliao
 * de implementaes do Ginga. Ela define o comportamento que ser apresentado
 * pelas aplicaes em caso dos testes serem bem sucedidos ou no. Os Xlets de
 * teste devem manter relacionamento com esta classe, a fim de se adequarem ao 
 * comportamento padro esperado nos casos mencionados. 
 *  
 * @author Bruno Pessoa
 * @author Gilberto Farias
 * @author Danilo Formiga
 *
 */
public class DTVTestLog extends Thread {
	
	public static final long TIMEOUT = 15000;

	// Utilizado para encerrar a aplicao, caso seu tempo limite seja alcanado.
	private long millis;
		    		
    private boolean flagLog = false;
    private List listeners;
    
    private int status;
    
    public DTVTestLog(long timeout, int executionModeID) {
    	status = executionModeID;
    	listeners = new ArrayList();           
        millis = timeout;
    }
    
    public void run() {    	
    	try {
			Thread.sleep(millis);
		} catch (InterruptedException e) {
			return;			
		}    	
    	
    	timeout();    	
    }

    public void addDTVTestEventListener(DTVTestListener l) {
        listeners.add(l);
    }

    public void removeDTVTestEventListener(DTVTestListener l) {
        listeners.remove(l);
    }

    private void notifyEventReceived(final DTVTestEvent e) {
    	
    	Iterator it = listeners.iterator();   	
        while (it.hasNext()) {
        	final DTVTestListener l = (DTVTestListener) it.next();        	
            new Thread() {

                public void run() {
                    l.testLogEventReceived(e);
                }
            }.start();
        }
    }        

    /**
     * Define o comportamento das aplicaes diante do sucesso na execuo dos 
     * testes realizados. 
     * @throws Exception 
     */
    public void success() {
    	
        if (flagLog == true) {
            return;
        }

        flagLog = true;
        status = DTVTestEvent.TEST_SUCCESS;
        notifyEventReceived(new DTVTestEvent(status)); 
        
    }

    /**
     * Define o comportamento das aplicaes em caso dos testes no serem bem
     * sucedidos. 
     */
    public void failure() {
    	
    	if (flagLog == true) {
            return;
        }
    	
        flagLog = true;
        
        status = DTVTestEvent.TEST_FAILURE;
        notifyEventReceived(new DTVTestEvent(status));  
    }

    /**
     * Determina o comportamento a ser apresentado pelas aplicaes de teste,
     * quando o tempo limite para sua execuo for alcanado.
     */
    private void timeout() {
    	
        if (flagLog == true) {
            return;
        }
        
        flagLog = true;
        
        status = DTVTestEvent.TEST_TIMEOUT;
        notifyEventReceived(new DTVTestEvent(status));        
    }
    
    public void visualAppFinished() {
        if (flagLog == true) {
            return;
        }
        
        try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
        
        flagLog = true;
        
        status = DTVTestEvent.TEST_VISUAL_APP;
        notifyEventReceived(new DTVTestEvent(status));        
    }

    public void setTimeOut(int ms) {
        this.millis = ms;
    }
    
    public void destroy() {

    }
    
    public boolean getFlagLog() {
    	return flagLog;
    }
    
    public int getStatus() {
    	return status;
    }
    
    public long getTimeout() {
    	return millis;
    }
    
}
